@@ -100,10 +100,9 @@ package Spreadsheet::ParseExcel;
 require Exporter;
 use strict;
 use OLE::Storage_Lite;
-use vars qw($VERSION @ISA );
+use vars qw($VERSION @ISA);
 @ISA = qw(Exporter);
-$VERSION = '0.21'; # 
-my $oFmtClass;
+$VERSION = '0.22'; # 
 my @aColor =
 (
     '000000',   # 0x00
@@ -284,13 +283,12 @@ sub Parse($$;$) {
 
 #2. Ready for format
     if ($oWkFmt) {
-        $oFmtClass = $oWkFmt;
+        $oBook->{FmtClass} = $oWkFmt;
     }
     else {
         require Spreadsheet::ParseExcel::FmtDefault;
-        $oFmtClass = new Spreadsheet::ParseExcel::FmtDefault;
+        $oBook->{FmtClass} = new Spreadsheet::ParseExcel::FmtDefault;
     }
-    $oThis->{FmtClass} = $oWkFmt;
 
 #3. Parse content
     my $lPos = 0;
@@ -519,7 +517,7 @@ sub _subRString($$$$)
             Numeric => 0,
             Code    => undef,
             Book    => $oBook,
-            STRUN   => substr($sWk, (8+$iL)+1),
+            Rich    => substr($sWk, (8+$iL)+1),
         );
     }
     else {
@@ -664,7 +662,7 @@ sub _subString($$$$)
 
     my ($iLen, $sTxt, $iCode);
     if($oBook->{BIFFVersion} == verBIFF8) {
-        my( $raBuff, $iLen) = _convBIFF8String($sWk, 1);
+        my( $raBuff, $iLen) = _convBIFF8String($oBook, $sWk, 1);
         $sTxt  = $raBuff->[0];
         $iCode = $raBuff->[1];
     }
@@ -702,7 +700,7 @@ sub _subLabel($$$$)
     #BIFF8
     if($oBook->{BIFFVersion} >= verBIFF8) {
         my ( $raBuff, $iLen, $iStPos, $iLenS) = 
-                _convBIFF8String(substr($sWk,6), 1);
+                _convBIFF8String($oBook, substr($sWk,6), 1);
         $sLbl  = $raBuff->[0];
         $sCode = ($raBuff->[1])? 'ucs2': undef;
     }
@@ -796,6 +794,7 @@ sub _subLabelSST($$$$)
             Numeric => 0,
             Code    => ($oBook->{PkgStr}[$iIdx]->{Unicode})? 'ucs2': undef,
             Book    => $oBook,
+            Rich   => $oBook->{PkgStr}[$iIdx]->{Rich},
         );
 
 #2.MaxRow, MaxCol, MinRow, MinCol
@@ -868,7 +867,7 @@ sub _subStandardWidth($$$$)
 {
     my($oBook, $bOp, $bLen, $sWk) = @_;
     my $iW = unpack("v", $sWk);
-    $oBook->{StandardWidth}= _adjustColWidth($iW);
+    $oBook->{StandardWidth}= _adjustColWidth($oBook, $iW);
 }
 #------------------------------------------------------------------------------
 # _subDefColWidth(for Spreadsheet::ParseExcel)      DK:P319
@@ -877,14 +876,16 @@ sub _subDefColWidth($$$$)
 {
     my($oBook, $bOp, $bLen, $sWk) = @_;
     my $iW = unpack("v", $sWk);
-    $oBook->{Worksheet}[$oBook->{_CurSheet}]->{DefColWidth}= _adjustColWidth($iW);
+    $oBook->{Worksheet}[$oBook->{_CurSheet}]->{DefColWidth}= _adjustColWidth($oBook, $iW);
 }
 #------------------------------------------------------------------------------
 # _adjustColWidth (for Spreadsheet::ParseExcel)
 #------------------------------------------------------------------------------
-sub _adjustColWidth($) {
-    my($iW)=@_;
-    return ($iW -0xA0)/256;
+sub _adjustColWidth($$) {
+    my($oBook, $iW)=@_;
+    return (($iW -0xA0)/256);
+#    ($oBook->{Worksheet}[$oBook->{_CurSheet}]->{SheetVersion} == verExcel97)?
+#        (($iW -0xA0)/256) : $iW;
 }
 #------------------------------------------------------------------------------
 # _subColInfo (for Spreadsheet::ParseExcel) DK:P309
@@ -894,7 +895,7 @@ sub _subColInfo($$$$)
     my($oBook, $bOp, $bLen, $sWk) = @_;
     my($iSc, $iEc, $iW, $iXF, $iGr) = unpack("v5", $sWk);
     for(my $i= $iSc; $i<=$iEc; $i++) {
-        $oBook->{Worksheet}[$oBook->{_CurSheet}]->{ColWidth}[$i] = _adjustColWidth($iW);
+        $oBook->{Worksheet}[$oBook->{_CurSheet}]->{ColWidth}[$i] = _adjustColWidth($oBook, $iW);
         $oBook->{Worksheet}[$oBook->{_CurSheet}]->{ColFmtNo}[$i] = $iXF;
         # $oBook->{Worksheet}[$oBook->{_CurSheet}]->{ColCr}[$i]    = $iGr; #Not Implemented
     }
@@ -930,19 +931,19 @@ sub _subWriteAccess($$$$)
 
     #BIFF8
     if($oBook->{BIFFVersion} >= verBIFF8) {
-        $oBook->{Author} = _convBIFF8String($sWk);
+        $oBook->{Author} = _convBIFF8String($oBook, $sWk);
     }
     #Before BIFF8
     else {
         my($iLen) = unpack("c", $sWk);
-        $oBook->{Author} = $oFmtClass->TextFmt(substr($sWk, 1, $iLen), '_native_');
+        $oBook->{Author} = $oBook->{FmtClass}->TextFmt(substr($sWk, 1, $iLen), '_native_');
     }
 }
 #------------------------------------------------------------------------------
 # _convBIFF8String (for Spreadsheet::ParseExcel)
 #------------------------------------------------------------------------------
-sub _convBIFF8String($;$){
-    my($sWk, $iCnvFlg) = @_;
+sub _convBIFF8String($$;$){
+    my($oBook, $sWk, $iCnvFlg) = @_;
     my($iLen, $iFlg) = unpack("vc", $sWk);
     my($iHigh, $iExt, $iRich) = ($iFlg & 0x01, $iFlg & 0x04, $iFlg & 0x08);
     my($iStPos, $iExtCnt, $iRichCnt, $sStr);
@@ -971,7 +972,7 @@ sub _convBIFF8String($;$){
         $iLen *= 2;
         $sStr = substr($sWk,    $iStPos, $iLen);
         _SwapForUnicode(\$sStr);
-        $sStr = $oFmtClass->TextFmt($sStr, 'ucs2') unless($iCnvFlg);
+        $sStr = $oBook->{FmtClass}->TextFmt($sStr, 'ucs2') unless($iCnvFlg);
     }
     else {              #Not Compressed
         $sStr = substr($sWk, $iStPos, $iLen);
@@ -1121,10 +1122,10 @@ sub _subFormat($$$$)
         ($oBook->{BIFFVersion} == verBIFF4) ||
         ($oBook->{BIFFVersion} == verBIFF5) ) {
         $sFmt = substr($sWk, 3, unpack('c', substr($sWk, 2, 1)));
-        $sFmt = $oFmtClass->TextFmt($sFmt, '_native_');
+        $sFmt = $oBook->{FmtClass}->TextFmt($sFmt, '_native_');
     }
     else {
-        $sFmt = _convBIFF8String(substr($sWk, 2));
+        $sFmt = _convBIFF8String($oBook, substr($sWk, 2));
     }
 #print "FMT:$sFmt\n";
     $oBook->{FormatStr}->{unpack('v', substr($sWk, 0, 2))} = $sFmt;
@@ -1154,7 +1155,7 @@ sub _subFont($$$$)
         $iUnicode = 1;
         $sFntName = substr($sWk, 16);
         _SwapForUnicode(\$sFntName);
-        $sFntName = $oFmtClass->TextFmt($sFntName, 'ucs2');
+        $sFntName = $oBook->{FmtClass}->TextFmt($sFntName, 'ucs2');
         $bBold       = ($iBold >= 0x2BC)? 1: 0;
         $bItalic     = ($iAttr & 0x02)? 1: 0;
         $bStrikeout  = ($iAttr & 0x08)? 1: 0;
@@ -1163,7 +1164,7 @@ sub _subFont($$$$)
     elsif($oBook->{BIFFVersion} == verBIFF5) {
         ($iHeight, $iAttr, $iCIdx, $iBold, $iSuper, $iUnderline) = 
             unpack("v5c", $sWk);
-        $sFntName = $oFmtClass->TextFmt(
+        $sFntName = $oBook->{FmtClass}->TextFmt(
                     substr($sWk, 15, unpack("c", substr($sWk, 14, 1))), 
                     '_native_');
         $bBold       = ($iBold >= 0x2BC)? 1: 0;
@@ -1214,7 +1215,7 @@ sub _subBoundSheet($$$$)
         my $sWsName = substr($sWk, 8);
         if($iUni & 0x01) {
             _SwapForUnicode(\$sWsName);
-            $sWsName = $oFmtClass->TextFmt($sWsName, 'ucs2');
+            $sWsName = $oBook->{FmtClass}->TextFmt($sWsName, 'ucs2');
         }
         $oBook->{Worksheet}[$oBook->{SheetCount}] = 
             new Spreadsheet::ParseExcel::Worksheet(
@@ -1226,7 +1227,7 @@ sub _subBoundSheet($$$$)
     else {
         $oBook->{Worksheet}[$oBook->{SheetCount}] = 
             new Spreadsheet::ParseExcel::Worksheet(
-                    Name => $oFmtClass->TextFmt(substr($sWk, 7), '_native_'),
+                    Name => $oBook->{FmtClass}->TextFmt(substr($sWk, 7), '_native_'),
                     Kind => $iKind,
                     _Pos => $iPos,
                 );
@@ -1242,13 +1243,13 @@ sub _subHeader($$$$)
     #BIFF8
     if($oBook->{BIFFVersion} >= verBIFF8) {
         $oBook->{Worksheet}[$oBook->{_CurSheet}]->{Header} = 
-            _convBIFF8String($sWk);
+            _convBIFF8String($oBook, $sWk);
     }
     #Before BIFF8
     else {
         my($iLen) = unpack("c", $sWk);
         $oBook->{Worksheet}[$oBook->{_CurSheet}]->{Header} 
-            = $oFmtClass->TextFmt(substr($sWk, 1, $iLen), '_native_');
+            = $oBook->{FmtClass}->TextFmt(substr($sWk, 1, $iLen), '_native_');
     }
 }
 #------------------------------------------------------------------------------
@@ -1260,13 +1261,13 @@ sub _subFooter($$$$)
     #BIFF8
     if($oBook->{BIFFVersion} >= verBIFF8) {
         $oBook->{Worksheet}[$oBook->{_CurSheet}]->{Footer} = 
-            _convBIFF8String($sWk);
+            _convBIFF8String($oBook, $sWk);
     }
     #Before BIFF8
     else {
         my($iLen) = unpack("c", $sWk);
         $oBook->{Worksheet}[$oBook->{_CurSheet}]->{Footer}
-            = $oFmtClass->TextFmt(substr($sWk, 1, $iLen), '_native_');
+            = $oBook->{FmtClass}->TextFmt(substr($sWk, 1, $iLen), '_native_');
     }
 }
 #------------------------------------------------------------------------------
@@ -1565,7 +1566,7 @@ sub _subMergeArea($$$$)
     my($oBook, $bOp, $bLen, $sWk) = @_;
     my $iCnt = unpack("v", $sWk);
     my $oWkS = $oBook->{Worksheet}[$oBook->{_CurSheet}];
-    $oWkS->{MergedArea} = () unless(defined $oWkS->{MergedAread});
+    $oWkS->{MergedArea} = [] unless(defined $oWkS->{MergedArea});
     for(my $i=0; $i < $iCnt; $i++) {
         my($iRs, $iRe, $iCs, $iCe) = unpack('n4', substr($sWk, $i*8 + 1, 8));
         for(my $iR=$iRs;$iR<=$iRe;$iR++) {
@@ -1709,7 +1710,7 @@ sub _subStrWk($$;$)
     $oBook->{_PrevInfo} = undef;
 
     while(length($oBook->{StrBuff}) >= 4) {
-        my ( $raBuff, $iLen, $iStPos, $iLenS) = _convBIFF8String($oBook->{StrBuff}, 1);
+        my ( $raBuff, $iLen, $iStPos, $iLenS) = _convBIFF8String($oBook, $oBook->{StrBuff}, 1);
                                                     #No Code Convert
         if(defined($raBuff->[0])) {
             push @{$oBook->{PkgStr}}, 
@@ -1755,13 +1756,22 @@ sub _NewCell($$$%)
             FormatNo=> $rhKey{FormatNo},
             Format  => $rhKey{Format},
             Code    => $rhKey{Code},
-            Type    => $oFmtClass->ChkType(
+            Type    => $oBook->{FmtClass}->ChkType(
                             $rhKey{Numeric}, 
                             $rhKey{Format}->{FmtIdx}),
         );
-    $oCell->{_Kind} = $rhKey{Kind};
-    $oCell->{_Value} = $oFmtClass->ValFmt($oCell, $rhKey{Book});
-    $oCell->{STRUN} = $rhKey{STRUN};
+    $oCell->{_Kind}  = $rhKey{Kind};
+    $oCell->{_Value} = $oBook->{FmtClass}->ValFmt($oCell, $oBook);
+
+    if($rhKey{Rich}) {
+        my @aRich = ();
+        my $sRich = $rhKey{Rich};
+        for(my $iWk=0;$iWk<length($sRich); $iWk+=4) {
+            my($iPos, $iFnt) = unpack('v2', substr($sRich, $iWk));
+            push @aRich, [$iPos, $oBook->{Font}[$iFnt]];
+        }
+        $oCell->{Rich}   =  \@aRich;
+    }
 
     if(defined $_CellHandler) {
         $_CellHandler->($oBook, $oBook->{_CurSheet}, $iR, $iC, $oCell);
@@ -2060,7 +2070,12 @@ Array ref of horizontal page breaks.
 
 =item VPageBreak
 
-Array fef of vertical page breaks.
+Array ref of vertical page breaks.
+
+=item MergedArea
+
+Array ref of merged areas.
+Each merged area is : [ I<StartRow>, I<StartColumn>, I<EndRow>, I<EndColumn>]
 
 =back
 
@@ -2099,6 +2114,14 @@ L<"Format"> for that cell.
 
 That cells is merged (or not).
 
+=item Rich
+
+Array ref of font informations about each characters.
+
+Each entry has : [ I<Start Position>, I<Font Object>]
+
+For more information please refer sample/dmpExR.pl
+
 =back
 
 =head2 Format
